import Products, Globals, OFS
from Products.ZCatalog import ZCatalog
from Persistence import Persistent
from Acquisition import Implicit
from Globals import HTMLFile
from DateTime import DateTime

def manage_addZDTopic(self, id, title, REQUEST=None):
    """Add a ZDTopic object"""

    topic = ZDTopic(id, title)
    self._setObject(id, topic)
    if REQUEST is not None:
	return self.manage_main(self, REQUEST)

class ZDUser(Implicit, Persistent):
    """This is a basic, stupid ZDUser class."""

    def __init__(self, author, email):
	self.author = author
	self.email = email

class ZDItem(OFS.SimpleItem.SimpleItem):
    """ZDItem object"""

    meta_type = "Z Discussions Item"

    def __init__(self, id, title, body, user, in_reply_to):
	self.id = id
	self.title = title
	self.user = user
	self.body = body
	self.in_reply_to = in_reply_to
	self.created = self.modified = DateTime()

    def ancestors(self):
	ancestors = []
	irp = self.in_reply_to
	while irp:
	    parent = getattr(self.aq_parent, irp)
	    ancestors = ancestors = [parent]
	    irp = parent.in_reply_to
	return ancestors

    def children(self, REQUEST=None):
	if REQUEST is None:
	    REQUEST = self.REQUEST
	children = []
	results = self.searchResults(in_reply_to=self.id) # XXX sort
	children = map(lambda x, self=self, REQUEST=REQUEST:
		        self.getobject(x.data_record_id_, REQUEST),
		       results)
	return children

    def manage_addReply(self, id, title, body, user=None, REQUEST=None):
	"""Add a ZDItem to this ZDTopic"""
    
	item = ZDItem(id, title, body, user, self.id)
	self.aq_parent._setObject(id, item)
	if REQUEST is not None:
	    return self.manage_main(self, REQUEST)

    def manage_afterAdd(self, item, container):
	OFS.SimpleItem.SimpleItem.manage_afterAdd(self, item, container)
	self.catalog_object(self, self.absolute_url(1))

    def manage_afterClone(self, item, container):
	OFS.SimpleItem.SimpleItem.manage_afterClone(self, item, container)
	self.catalog_object(self, self.absolute_url(1))

    def manage_beforeDelete(self, item, container):
	OFS.SimpleItem.SimpleItem.manage_beforeAdd(self, item, container)
	self.uncatalog_object(self.absolute_url(1))

class ZDTopic(ZCatalog.ZCatalog):
    """ZDTopic object"""

    meta_type = 'Z Discussions Topic'
    meta_types = (
	{'name': ZDItem.meta_type,
	 'action':'manage_addZDItemForm'}
	)

    manage_options=(
        {'label': 'Contents', 'action': 'manage_main', 'target': 'manage_main'},
        {'label': 'Cataloged Objects', 'action': 'manage_catalogView', 'target': 'manage_main'},
        {'label': 'MetaData Table', 'action': 'manage_catalogSchema', 'target':'manage_main'},
        {'label': 'Indexes', 'action': 'manage_catalogIndexes', 'target':'manage_main'},
        {'label': 'Status', 'action': 'manage_catalogStatus', 'target':'manage_main'},
        )

    def __init__(self, id, title='', UserClass=ZDUser):

	# Initialise ZCatalog
	ZCatalog.ZCatalog.__init__(self, id, title)

	# Set up the indexes
	self._catalog.addIndex('body', 'TextIndex')
	self._catalog.addIndex('author', 'TextIndex')  # Arguably, this could be FieldIndex
	self._catalog.addIndex('email', 'TextIndex') # Ditto
	self._catalog.addIndex('in_reply_to', 'FieldIndex') # Do I want FieldIndex?
	self._catalog.addIndex('created', 'TextIndex')
	self._catalog.addIndex('modified', 'TextIndex')

	# Set up meta-data columns
	self._catalog.addColumn('body')
	self._catalog.addColumn('author')
	self._catalog.addColumn('email')
	self._catalog.addColumn('in_reply_to')
	self._catalog.addColumn('created')
	self._catalog.addColumn('modified')
	
	self.UserClass = UserClass

    def manage_editZDTopic(self, title, REQUEST=None):
	"""Set properties of ZDTopic.  Presently just title."""

	self.title = title
	if REQUEST:
	    return self.manage_main(self, REQUEST, manage_tabs_message = "ZDTopic changed.")

    def posts(self, REQUEST=None):
	if REQUEST is None:
	    REQUEST=self.REQUEST
	posts = []
	for res in self.searchResults(in_reply_to=''): #, sort_on='created'):
	    posts.append(self.getobject(res.data_record_id_, REQUEST))
	return posts

    def manage_addZDItem(self, id, title, body, user=None, in_reply_to='', REQUEST=None):
	"""Add a ZDItem to this ZDTopic"""
    
	item = ZDItem(id, title, body, user, in_reply_to)
	self._setObject(id, item)
	if REQUEST is not None:
	    return self.manage_main(self, REQUEST)
